/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Alphex Kaanoken <kaanoken@crew.org.ru>
 *
 *  (c) Crew IT Research Labs 2004
 *
 */

#include "detection.h"
#include "twolpoint.h"
#include "weight.h"
#include "dict.h"

/*this functions determine the symbol (a|b)*/
/*NOTE: This function generated automatically by some my mathematical model*/
int isAB(char byte)
{
  switch(byte)
    {
    case 'a':
      return 1;
      break;
    case 'b':
      return 0;
      break;
    case 'c':
      return 0;
      break;
    case 'd':
      return 0;
      break;
    case 'e':
      return 1;
      break;
    case 'f':
      return 0;
      break;
    case 'g':
      return 0;
      break;
    case 'h':
      return 0;
      break;
    case 'i':
      return 1;
      break;
    case 'j':
      return 0;
      break;
    case 'k':
      return 0;
      break;
    case 'l':
      return 0;
      break;
    case 'm':
      return 0;
      break;
    case 'n':
      return 0;
      break;
    case 'o':
      return 1;
      break;
    case 'p':
      return 0;
      break;
    case 'q':
      return 0;
      break;
    case 'r':
      return 0;
      break;
    case 's':
      return 0;
      break;
    case 't':
      return 0;
      break;
    case 'u':
      return 1;
      break;
    case 'v':
      return 0;
      break;
    case 'w':
      return 0;
      break;
    case 'x':
      return 0;
      break;
    case 'y':
      return 1;
      break;
    case 'z':
      return 0;
      break;
    default:
      return -1;
    }
}

/*this function determine this is Cyrillic word or not?
 * return percentage of this. (analysis of symbols)
 * Some part of this functions was generated automatically, 
 * and code looks not beatyfully, but it just works good and faster - it was 
 * was my target
 */
int isCyrWord(const char *word)
{
  int i=0;
  int lw=strlen(word);
  int points=0;
  int tk=0;
  int shr=0;
  if(lw<1)
    return 0; 
  if(lw==1)
    {
    switch(word[0]){
      case 'e':
        return 100;
	break;
      case 'r':
        return 100;
	break;
      case 'd':
        return 100;
	break;
      case 'f':
        return 100;
	break;
      case 'j':
        return 100;
	break;
      case 'z':
        return 100;
	break;
      case 'c':
        return 100;
	break;
      case 'b':
        return 100;
	break;
      default:
        return -1;
        break;   
      }
    }
  if(lw==2) /*simple 2 symbol word analysis*/
    {
      /*TODO: make a normal analysis with small words*/
      i=isAB(word[0])+isAB(word[1]);
      if(i==0)
	return -1;
      if(i<0)
	return 100;
    }
  /*detect for steps*/
  tk=lw/4;
  shr=lw%4;
  i=-1;
  if(tk>=1) {
    while(i!=shr) {
      if(!isAB(word[i+1])&&!isAB(word[i+2])&&!isAB(word[i+3])&&!isAB(word[i+4]))
	points+=60;
      if(isAB(word[i+1])&&isAB(word[i+2])&&isAB(word[i+3])&&isAB(word[i+4]))
	points+=60;
      ++i;
    }
  }
  /*stage one - we see for 4a|b in word, with moving*/
  i=0;
  while(i!=lw-1)
    {
      switch(word[i]){
      case '[':
	points+=80;
	break;
      case ']':
	points+=80;
	break;
      case '\'':
	points+=80;
	break;
      case ',':
	points+=80;
	break;
      case '.':
	points+=80;
	break;
      case ';':
	points+=80;
	break;
      default:
	break;
      }
      ++i;
    }
  /*stage two - we see for {}[];:'",.<> in word*/
  /*stage three lexical analysis
   * It's point was taken from output of my algorythm of searching for
   * slogs in english and russian. I'm use two dictionaryes
   */
  /*NOTE: this part generated automatically by my model, in future this model
   will be stored in CVS repositary and I add support for other languages 
  dynamically*/
  tk=lw/2;
  shr=lw%2;
  i=-1;
  if(tk>=1) {
    while(i!=shr) {
      switch(word[i+1]) {
	/*check 2l B*/
      case 'b':
	switch(word[i+2]){
	case 'c':
	  points+=BC;
	  break;
	case 'd':
	  points+=BD;
	  break;
	case 'f':
	  points+=BF;
	  break;
	case 'g':
	  points+=BG;
	  break;
	case 'h':
	  points+=BH;
	  break;
	case 'j':
	  points+=BJ;
	  break;
	case 'k':
	  points+=BK;
	  break;
	case 'n':
	  points+=BN;
	  break;
	case 'p':
	  points+=BP;
	  break;
	case 'q':
	  points+=BQ;
	  break;
	case 't':
	  points+=BT;
	  break;
	case 'v':
	  points+=BV;
	  break;
	case 'w':
	  points+=BW;
	  break;
	}
	break;
	/*check 2l C*/
      case 'c':
	switch(word[i+2]){
	case 'b':
	  points+=CB;
	  break;
	case 'd':
	  points+=CD;
	  break;
	case 'f':
	  points+=CF;
	  break;
	case 'g':
	  points+=CG;
	  break;
	case 'j':
	  points+=CJ;
	  break;
	case 'm':
	  points+=CM;
	  break;
	case 'n':
	  points+=CN;
	  break;
	case 'v':
	  points+=CV;
	  break;
	case 'w':
	  points+=CW;
	  break;
	case 'x':
	  points+=CX;
	  break;
	case 'z':
	  points+=CZ;
	  break;
	}
	break;
	/*check 2l D*/
      case 'd':
	switch(word[i+2]){
	case 'b':
	  points+=DB;
	  break;
	case 'c':
	  points+=DC;
	  break;
	case 'f':
	  points+=DF;
	  break;
	case 'h':
	  points+=DH;
	  break;
	case 'j':
	  points+=DJ;
	  break;
	case 'k':
	  points+=DK;
	  break;
	case 'n':
	  points+=DN;
	  break;
	case 'p':
	  points+=DP;
	  break;
	case 's':
	  points+=DS;
	  break;
	case 't':
	  points+=DT;
	  break;
	case 'x':
	  points+=DX;
	  break;
	case 'z':
	  points+=DZ;
	  break;
	}
	break;
	/*check 2l E*/
      case 'e':
	switch(word[i+2]){
	case 'h':
	  points+=EH;
	  break;
	case 'k':
	  points+=EK;
	  break;
	}
	break;
	/*check 2l F*/
      case 'f':
	switch(word[i+2]){
	case 'b':
	  points+=FB;
	  break;
	case 'c':
	  points+=FC;
	  break;
	case 'd':
	  points+=FD;
	  break;
	case 'g':
	  points+=FG;
	  break;
	case 'h':
	  points+=FH;
	  break;
	case 'j':
	  points+=FJ;
	  break;
	case 'k':
	  points+=FK;
	  break;
	case 'n':
	  points+=FN;
	  break;
	case 'p':
	  points+=FP;
	  break;
	case 'q':
	  points+=FQ;
	  break;
	case 'v':
	  points+=FV;
	  break;
	case 'w':
	  points+=FW;
	  break;
	case 'x':
	  points+=FX;
	  break;
	case 'y':
	  points+=FY;
	  break;
	case 'z':
	  points+=FZ;
	  break;
	}
	break;
	/*check 2l G*/
      case 'g':
	switch(word[i+2]){
	case 'b':
	  points+=GB;
	  break;
	case 'j':
	  points+=GJ;
	  break;
	case 'k':
	  points+=GK;
	  break;
	case 't':
	  points+=GT;
	  break;
	case 'x':
	  points+=GX;
	  break;
	case 'z':
	  points+=GZ;
	  break;
	}
	break;
	/*check 2l H*/
      case 'h':
	switch(word[i+2]){
	case 'b':
	  points+=HB;
	  break;
	case 'c':
	  points+=HC;
	  break;
	case 'd':
	  points+=HD;
	  break;
	case 'f':
	  points+=HF;
	  break;
	case 'g':
	  points+=HG;
	  break;
	case 'j':
	  points+=HJ;
	  break;
	case 'k':
	  points+=HK;
	  break;
	case 'l':
	  points+=HL;
	  break;
	case 'm':
	  points+=HM;
	  break;
	case 'n':
	  points+=HN;
	  break;
	case 's':
	  points+=HS;
	  break;
	case 'v':
	  points+=HV;
	  break;
	case 'x':
	  points+=HX;
	  break;
	case 'z':
	  points+=_HZ;
	  break;
	}
	break;
	/*check 2l J*/
      case 'j':
	switch(word[i+2]){
	case 'b':
	  points+=JB;
	  break;
	case 'c':
	  points+=JC;
	  break;
	case 'd':
	  points+=JD;
	  break;
	case 'f':
	  points+=JF;
	  break;
	case 'g':
	  points+=JG;
	  break;
	case 'h':
	  points+=JH;
	  break;
	case 'i':
	  points+=JI;
	  break;
	case 'j':
	  points+=JJ;
	  break;
	case 'k':
	  points+=JK;
	  break;
	case 'l':
	  points+=JL;
	  break;
	case 'n':
	  points+=JN;
	  break;
	case 'p':
	  points+=JP;
	  break;
	case 'q':
	  points+=JQ;
	  break;
	case 'r':
	  points+=JR;
	  break;
	case 't':
	  points+=JT;
	  break;
	case 'v':
	  points+=JV;
	  break;
	case 'w':
	  points+=JW;
	  break;
	case 'x':
	  points+=JX;
	  break;
	case 'y':
	  points+=JY;
	  break;
	case 'z':
	  points+=JZ;
	  break;
	}
	break;
	/*check 2l K*/
      case 'k':
	switch(word[i+2]){
	case 'b':
	  points+=KB;
	  break;
	case 'f':
	  points+=KF;
	  break;
	case 'j':
	  points+=KJ;
	  break;
	case 'k':
	  points+=KK;
	  break;
	case 'm':
	  points+=KM;
	  break;
	case 't':
	  points+=KT;
	  break;
	case 'v':
	  points+=KV;
	  break;
	case 'x':
	  points+=KX;
	  break;
	case 'z':
	  points+=KZ;
	  break;
	}
	break;
      case 'l':
	switch(word[i+2]){
	case 'j':
	  points+=LJ;
	  break;
	case 'x':
	  points+=LX;
	  break;
	case 'z':
	  points+=LZ;
	  break;
	}
	break;
	/*check 2l M*/
      case 'm':
	switch(word[i+2]){
	case 'c':
	  points+=MC;
	  break;
	case 'd':
	  points+=MD;
	  break;
	case 'v':
	  points+=MV;
	  break;
	case 'x':
	  points+=MX;
	  break;
	case 'z':
	  points+=MZ;
	  break;
	}
	break;
	/*check 2l N*/
      case 'n':
	switch(word[i+2]){
	case 'j':
	  points+=NJ;
	  break;
	case 'm':
	  points+=NM;
	  break;
	case 'z':
	  points+=NZ;
	  break;
	}
	break;
	/*check 2l P*/
      case 'p':
	switch(word[i+2]){
	case 'b':
	  points+=PB;
	  break;
	case 'd':
	  points+=PD;
	  break;
	case 'f':
	  points+=PF;
	  break;
	case 'j':
	  points+=PJ;
	  break;
	case 'k':
	  points+=PK;
	  break;
	case 'v':
	  points+=PV;
	  break;
	case 'x':
	  points+=PX;
	  break;
	case 'z':
	  points+=PZ;
	  break;
	}
	break;
	/*check 2l Q*/
      case 'q':
	switch(word[i+2]){
	case 'a':
	  points+=QA;
	  break;
	case 'c':
	  points+=QC;
	  break;
	case 'd':
	  points+=QD;
	  break;
	case 'g':
	  points+=QG;
	  break;
	case 'i':
	  points+=QI;
	  break;
	case 'j':
	  points+=QJ;
	  break;
	case 'k':
	  points+=QK;
	  break;
	case 'l':
	  points+=QL;
	  break;
	case 'n':
	  points+=QN;
	  break;
	case 'o':
	  points+=QO;
	  break;
	case 'r':
	  points+=QR;
	  break;
	case 'v':
	  points+=QV;
	  break;
	case 'w':
	  points+=QW;
	  break;
	case 'x':
	  points+=QX;
	  break;
	case 'y':
	  points+=QY;
	  break;
	case 'z':
	  points+=QZ;
	  break;
	}
	break;
	/*check 2l R*/
      case 'r':
	switch(word[i+2]){
	case 'f':
	  points+=RF;
	  break;
	case 'h':
	  points+=RH;
	  break;
	case 'j':
	  points+=RJ;
	  break;
	}
	break;
	/*check 2l S*/
      case 's':
	switch(word[i+2]){
	case 'd':
	  points+=SD;
	  break;
	case 'q':
	  points+=SQ;
	  break;
	case 'x':
	  points+=SX;
	  break;
	case 'z':
	  points+=SZ;
	  break;
	}
	break;
      case 't':
	switch(word[i+2]){
	case 'c':
	  points+=TC;
	  break;
	case 'd':
	  points+=TD;
	  break;
	case 'g':
	  points+=TG;
	  break;
	case 'j':
	  points+=TJ;
	  break;
	case 'k':
	  points+=TK;
	  break;
	case 'n':
	  points+=TN;
	  break;
	case 'p':
	  points+=TP;
	  break;
	case 'q':
	  points+=TQ;
	  break;
	case 'v':
	  points+=TV;
	  break;
	case 'x':
	  points+=TX;
	  break;
	}
	break;
	/*check 2l U*/
      case 'u':
	switch(word[i+2]){
	case 'h':
	  points+=UH;
	  break;
	case 'j':
	  points+=UJ;
	  break;
	case 'k':
	  points+=UK;
	  break;
	}
	break;
	/*check 2l V*/
      case 'v':
	switch(word[i+2]){
	case 'b':
	  points+=VB;
	  break;
	case 'c':
	  points+=VC;
	  break;
	case 'd':
	  points+=VD;
	  break;
	case 'f':
	  points+=VF;
	  break;
	case 'g':
	  points+=VG;
	  break;
	case 'h':
	  points+=VH;
	  break;
	case 'j':
	  points+=VJ;
	  break;
	case 'k':
	  points+=VK;
	  break;
	case 'r':
	  points+=VR;
	  break;
	case 's':
	  points+=VS;
	  break;
	case 't':
	  points+=VT;
	  break;
	case 'z':
	  points+=VZ;
	  break;
	}
	break;
	/*check 2l W*/
      case 'w':
	switch(word[i+2]){
	case 'b':
	  points+=WB;
	  break;
	case 'j':
	  points+=WJ;
	  break;
	case 't':
	  points+=WT;
	  break;
	}
	break;
	/*check 2l X*/
      case 'x':
	switch(word[i+2]){
	case 'b':
	  points+=XB;
	  break;
	case 'f':
	  points+=XF;
	  break;
	case 'j':
	  points+=XJ;
	  break;
	case 'k':
	  points+=XK;
	  break;
	case 'm':
	  points+=XM;
	  break;
	case 'n':
	  points+=XN;
	  break;
	case 'r':
	  points+=XR;
	  break;
	case 't':
	  points+=XT;
	  break;
	case 'v':
	  points+=XV;
	  break;
	case 'y':
	  points+=XY;
	  break;
	}
	break;
	/*check 2l Y*/
      case 'y':
	switch(word[i+2]){
	case 'b':
	  points+=YB;
	  break;
	case 'f':
	  points+=YF;
	  break;
	case 'j':
	  points+=YJ;
	  break;
	case 't':
	  points+=YT;
	  break;
	case 'u':
	  points+=YU;
	  break;
	case 'y':
	  points+=YY;
	  break;
	case 'z':
	  points+=YZ;
	  break;
	}
	break;
	/*check 2l Z*/
      case 'z':
	switch(word[i+2]){
	case 'c':
	  points+=ZC;
	  break;
	case 'd':
	  points+=ZD;
	  break;
	case 'h':
	  points+=ZH;
	  break;
	case 'n':
	  points+=ZN;
	  break;
	case 'p':
	  points+=ZP;
	  break;
	case 'q':
	  points+=ZQ;
	  break;
	case 'r':
	  points+=ZR;
	  break;
	case 't':
	  points+=ZT;
	  break;
	case 'u':
	  points+=ZU;
	  break;
	case 'v':
	  points+=ZV;
	  break;
	case 'w':
	  points+=ZW;
	  break;
	case 'x':
	  points+=ZX;
	  break;
	case 'y':
	  points+=ZY;
	  break;
	}
	break;
      }
      ++i;
    }
  }
  return points;
}



/*Noting in english*/

/*this new function determine this is Cyrillic word or not?
 * return > 0 if eng, or < 0 if rus. (analysis of symbols)
 * Some part of this functions was generated automatically, 
 * and code looks not beatyfully, but it just works good and faster - it was 
 * was my target
 */
int WhatLang(const char *word)
{
  char *notengword=DICTIONARY;
  char wordstr[5];
  int i=0;
  int lw=strlen(word);
  int points=0;
  int tk=0;
  int weight=0;
  char *shr;
  if(lw<1)
    return 0; 
  if(lw==1){
    switch(word[0]) {
      // RU
      case 'e':
      case 'r':
      case 'd':
      case 'f':
      case 'j':
      case 'z':
      case 'c':
      case 'b':
        return 1;
	break;
      case 'a':
        return -1;
        break;
      default:
        return 0;
        break;   
    }
  }
  if(lw==2){
    if ((word[1] == '.')||(word[1] == ',')) {
      return 1;
    }
    else {
      weight = GetWeigthSyllable(word[0], word[1]);
      if (weight > 0){ // is russian
        return 1;
      }
      else if (weight < 0){ // is english
        return -1;
      }
      else
        return 0;
    }
  }
  if(lw==3){
    if (word[2] == ' ') {
      weight = GetWeigthSyllable(word[0], word[1]);
      if (weight > 0){ // is russian
        return 1;
      }
      else if (weight < 0){ // is english
        return -1;
      }
      else
        return 0;
    }
    else {
      wordstr[0] = '|';
      wordstr[1] = word[0];
      wordstr[2] = word[1];
      wordstr[3] = word[2];
      wordstr[4] = '|';
      wordstr[5] = '\0';
      shr = strstr(notengword, wordstr);
      if (shr != NULL) {
	return 1;
      }
      else {
        for (i = 0; i <= (lw -1); i++){ 
          weight = weight + GetWeigthSyllable(word[i], word[i+1]);
        }
        if (weight > 0){ // is russian
          return 1;
        }
        else if (weight < 0){ // is english
          return -1;
        }
        else
          return 0;
      }
    }
  }
  if(lw>=4){
    i=0;
    do {      
      wordstr[0] = '|';
      wordstr[1] = word[i];
      wordstr[2] = word[i+1];
      wordstr[3] = word[i+2];
      wordstr[4] = '|';
      wordstr[5] = '\0';
      shr = strstr(notengword, wordstr);
      if (shr != NULL) {
        return 1;
        break;
      }
      i=i+1;
    } while (i <= (lw - 3));
    if (shr == NULL){
      for (i = 0; i <= (lw -1); i++){ 
        weight = weight + GetWeigthSyllable(word[i], word[i+1]);
      }
      if (weight > 0){ // is russian
        return 1;
      }
      else if (weight < 0){ // is english
        return -1;
      }
      else
        return 0;
    }
    return 0;
    
  }
  return 0;
}

// Function Return Weigth Syllable
int GetWeigthSyllable(char word1, char word2){
int i = 0;
int weight  = 0;
switch(word1){
  case 'q':
    switch(word2){
    case 'r':
      weight += __QR;
      break;
    case 'y':
      weight += __QY;
      break;
    case 'u':
      weight += __QU;
      break;
    case 'i':
      weight += __QI;
      break;
    case 'l':
      weight += __QL;
      break;
    case 'x':
      weight += __QX;
      break;
    case 'c':
      weight += __QC;
      break;
    case 'v':
      weight += __QV;
      break;
    case 'n':
      weight += __QN;
      break;
    case ',':
      weight += __Qzp;
      break;
    case '.':
      weight += __Qtch;
      break;
    }
  break;
  case 'w':
    switch(word2){
    case 'e':
      weight += __WE;
      break;
    case 't':
      weight += __WT;
      break;
    case 'i':
      weight += __WI;
      break;
    case 'o':
      weight += __WO;
      break;
    case 'a':
      weight += __WA;
      break;
    case 'f':
      weight += __WF;
      break;
    case 'h':
      weight += __WH;
      break;
    case 'j':
      weight += __WJ;
      break;
    case 'l':
      weight += __WL;
      break;
    case 'b':
      weight += __WB;
      break;
    case 'n':
      weight += __WN;
      break;
    case ',':
      weight += __Wzp;
      break;
    case '.':
      weight += __Wtch;
      break;
    }
  break;
  case 'e':
    switch(word2){
    case 'q':
      weight += __EQ;
      break;
    case 'w':
      weight += __EW;
      break;
    case 'e':
      weight += __EE;
      break;
    case 'r':
      weight += __ER;
      break;
    case 't':
      weight += __ET;
      break;
    case 'y':
      weight += __EY;
      break;
    case 'u':
      weight += __EU;
      break;
    case 'i':
      weight += __EI;
      break;
    case 'p':
      weight += __EP;
      break;
    case 'a':
      weight += __EA;
      break;
    case 's':
      weight += __ES;
      break;
    case 'd':
      weight += __ED;
      break;
    case 'f':
      weight += __EF;
      break;
    case 'g':
      weight += __EG;
      break;
    case 'h':
      weight += __EH;
      break;
    case 'k':
      weight += __EK;
      break;
    case 'l':
      weight += __EL;
      break;
    case 'z':
      weight += __EZ;
      break;
    case 'x':
      weight += __EX;
      break;
    case 'c':
      weight += __EC;
      break;
    case 'v':
      weight += __EV;
      break;
    case 'n':
      weight += __EN;
      break;
    case 'm':
      weight += __EM;
      break;
    case '[':
      weight += __Eksl;
      break;
    case ';':
      weight += __Etz;
      break;
    case ',':
      weight += __Ezp;
      break;
    case '.':
      weight += __Etch;
      break;
    }
  break;
  case 'r':
    switch(word2){
    case 'e':
      weight += __RE;
      break;
    case 'r':
      weight += __RR;
      break;
    case 't':
      weight += __RT;
      break;
    case 'y':
      weight += __RY;
      break;
    case 'u':
      weight += __RU;
      break;
    case 'i':
      weight += __RI;
      break;
    case 'o':
      weight += __RO;
      break;
    case 'p':
      weight += __RP;
      break;
    case 'a':
      weight += __RA;
      break;
    case 's':
      weight += __RS;
      break;
    case 'd':
      weight += __RD;
      break;
    case 'f':
      weight += __RF;
      break;
    case 'g':
      weight += __RG;
      break;
    case 'h':
      weight += __RH;
      break;
    case 'j':
      weight += __RJ;
      break;
    case 'l':
      weight += __RL;
      break;
    case 'c':
      weight += __RC;
      break;
    case 'v':
      weight += __RV;
      break;
    case 'b':
      weight += __RB;
      break;
    case 'n':
      weight += __RN;
      break;
    case 'm':
      weight += __RM;
      break;
    case ';':
      weight += __Rtz;
      break;
    case ',':
      weight += __Rzp;
      break;
    case '.':
      weight += __Rtch;
      break;
    }
  break;
  case 't':
    switch(word2){
    case 'q':
      weight += __TQ;
      break;
    case 'w':
      weight += __TW;
      break;
    case 'e':
      weight += __TE;
      break;
    case 'r':
      weight += __TR;
      break;
    case 't':
      weight += __TT;
      break;
    case 'y':
      weight += __TY;
      break;
    case 'u':
      weight += __TU;
      break;
    case 'i':
      weight += __TI;
      break;
    case 'o':
      weight += __TO;
      break;
    case 'p':
      weight += __TP;
      break;
    case 'a':
      weight += __TA;
      break;
    case 's':
      weight += __TS;
      break;
    case 'd':
      weight += __TD;
      break;
    case 'g':
      weight += __TG;
      break;
    case 'h':
      weight += __TH;
      break;
    case 'j':
      weight += __TJ;
      break;
    case 'k':
      weight += __TK;
      break;
    case 'l':
      weight += __TL;
      break;
    case 'z':
      weight += __TZ;
      break;
    case 'x':
      weight += __TX;
      break;
    case 'c':
      weight += __TC;
      break;
    case 'v':
      weight += __TV;
      break;
    case 'b':
      weight += __TB;
      break;
    case 'n':
      weight += __TN;
      break;
    case 'm':
      weight += __TM;
      break;
    case '[':
      weight += __Tksl;
      break;
    case ';':
      weight += __Ttz;
      break;
    case ',':
      weight += __Tzp;
      break;
    case '.':
      weight += __Ttch;
      break;
    }
  break;
  case 'y':
    switch(word2){
    case 'w':
      weight += __YW;
      break;
    case 'e':
      weight += __YE;
      break;
    case 'r':
      weight += __YR;
      break;
    case 't':
      weight += __YT;
      break;
    case 'y':
      weight += __YY;
      break;
    case 'u':
      weight += __YU;
      break;
    case 'i':
      weight += __YI;
      break;
    case 'o':
      weight += __YO;
      break;
    case 's':
      weight += __YS;
      break;
    case 'd':
      weight += __YD;
      break;
    case 'f':
      weight += __YF;
      break;
    case 'h':
      weight += __YH;
      break;
    case 'j':
      weight += __YJ;
      break;
    case 'l':
      weight += __YL;
      break;
    case 'z':
      weight += __YZ;
      break;
    case 'x':
      weight += __YX;
      break;
    case 'c':
      weight += __YC;
      break;
    case 'b':
      weight += __YB;
      break;
    case 'n':
      weight += __YN;
      break;
    case 'm':
      weight += __YM;
      break;
    case ';':
      weight += __Ytz;
      break;
    case ',':
      weight += __Yzp;
      break;
    case '.':
      weight += __Ytch;
      break;
    }
  break;
  case 'u':
    switch(word2){
    case 'e':
      weight += __UE;
      break;
    case 'r':
      weight += __UR;
      break;
    case 't':
      weight += __UT;
      break;
    case 'y':
      weight += __UY;
      break;
    case 'i':
      weight += __UI;
      break;
    case 'p':
      weight += __UP;
      break;
    case 'a':
      weight += __UA;
      break;
    case 's':
      weight += __US;
      break;
    case 'd':
      weight += __UD;
      break;
    case 'f':
      weight += __UF;
      break;
    case 'g':
      weight += __UG;
      break;
    case 'h':
      weight += __UH;
      break;
    case 'j':
      weight += __UJ;
      break;
    case 'k':
      weight += __UK;
      break;
    case 'l':
      weight += __UL;
      break;
    case 'c':
      weight += __UC;
      break;
    case 'b':
      weight += __UB;
      break;
    case 'n':
      weight += __UN;
      break;
    case 'm':
      weight += __UM;
      break;
    }
  break;
  case 'i':
    switch(word2){
    case 'e':
      weight += __IE;
      break;
    case 'r':
      weight += __IR;
      break;
    case 't':
      weight += __IT;
      break;
    case 'y':
      weight += __IY;
      break;
    case 'o':
      weight += __IO;
      break;
    case 'p':
      weight += __IP;
      break;
    case 'a':
      weight += __IA;
      break;
    case 's':
      weight += __IS;
      break;
    case 'd':
      weight += __ID;
      break;
    case 'f':
      weight += __IF;
      break;
    case 'g':
      weight += __IG;
      break;
    case 'j':
      weight += __IJ;
      break;
    case 'l':
      weight += __IL;
      break;
    case 'z':
      weight += __IZ;
      break;
    case 'x':
      weight += __IX;
      break;
    case 'c':
      weight += __IC;
      break;
    case 'v':
      weight += __IV;
      break;
    case 'b':
      weight += __IB;
      break;
    case 'n':
      weight += __IN;
      break;
    case 'm':
      weight += __IM;
      break;
    }
  break;
  case 'o':
    switch(word2){
    case 'w':
      weight += __OW;
      break;
    case 'e':
      weight += __OE;
      break;
    case 'r':
      weight += __OR;
      break;
    case 't':
      weight += __OT;
      break;
    case 'y':
      weight += __OY;
      break;
    case 'u':
      weight += __OU;
      break;
    case 'i':
      weight += __OI;
      break;
    case 'o':
      weight += __OO;
      break;
    case 'p':
      weight += __OP;
      break;
    case 'a':
      weight += __OA;
      break;
    case 's':
      weight += __OS;
      break;
    case 'd':
      weight += __OD;
      break;
    case 'f':
      weight += __OF;
      break;
    case 'g':
      weight += __OG;
      break;
    case 'h':
      weight += __OH;
      break;
    case 'k':
      weight += __OK;
      break;
    case 'l':
      weight += __OL;
      break;
    case 'c':
      weight += __OC;
      break;
    case 'v':
      weight += __OV;
      break;
    case 'b':
      weight += __OB;
      break;
    case 'n':
      weight += __ON;
      break;
    case 'm':
      weight += __OM;
      break;
    case ',':
      weight += __Ozp;
      break;
    case '.':
      weight += __Otch;
      break;
    }
  break;
  case 'p':
    switch(word2){
    case 'e':
      weight += __PE;
      break;
    case 'r':
      weight += __PR;
      break;
    case 't':
      weight += __PT;
      break;
    case 'y':
      weight += __PY;
      break;
    case 'i':
      weight += __PI;
      break;
    case 'o':
      weight += __PO;
      break;
    case 'p':
      weight += __PP;
      break;
    case 'a':
      weight += __PA;
      break;
    case 'd':
      weight += __PD;
      break;
    case 'f':
      weight += __PF;
      break;
    case 'h':
      weight += __PH;
      break;
    case 'j':
      weight += __PJ;
      break;
    case 'k':
      weight += __PK;
      break;
    case 'l':
      weight += __PL;
      break;
    case 'z':
      weight += __PZ;
      break;
    case 'v':
      weight += __PV;
      break;
    case 'b':
      weight += __PB;
      break;
    case 'm':
      weight += __PM;
      break;
    case ';':
      weight += __Ptz;
      break;
    case ',':
      weight += __Pzp;
      break;
    case '.':
      weight += __Ptch;
      break;
    }
  break;
  case 'a':
    switch(word2){
    case 'w':
      weight += __AW;
      break;
    case 'e':
      weight += __AE;
      break;
    case 'r':
      weight += __AR;
      break;
    case 't':
      weight += __AT;
      break;
    case 'y':
      weight += __AY;
      break;
    case 'u':
      weight += __AU;
      break;
    case 'i':
      weight += __AI;
      break;
    case 'p':
      weight += __AP;
      break;
    case 's':
      weight += __AS;
      break;
    case 'd':
      weight += __AD;
      break;
    case 'f':
      weight += __AF;
      break;
    case 'g':
      weight += __AG;
      break;
    case 'j':
      weight += __AJ;
      break;
    case 'k':
      weight += __AK;
      break;
    case 'l':
      weight += __AL;
      break;
    case 'z':
      weight += __AZ;
      break;
    case 'c':
      weight += __AC;
      break;
    case 'v':
      weight += __AV;
      break;
    case 'b':
      weight += __AB;
      break;
    case 'n':
      weight += __AN;
      break;
    case 'm':
      weight += __AM;
      break;
    case ',':
      weight += __Azp;
      break;
    case '.':
      weight += __Atch;
      break;
    }
  break;
  case 's':
    switch(word2){
    case 'q':
      weight += __SQ;
      break;
    case 'w':
      weight += __SW;
      break;
    case 'e':
      weight += __SE;
      break;
    case 'r':
      weight += __SR;
      break;
    case 't':
      weight += __ST;
      break;
    case 'u':
      weight += __SU;
      break;
    case 'i':
      weight += __SI;
      break;
    case 'o':
      weight += __SO;
      break;
    case 'p':
      weight += __SP;
      break;
    case 'a':
      weight += __SA;
      break;
    case 's':
      weight += __SS;
      break;
    case 'd':
      weight += __SD;
      break;
    case 'f':
      weight += __SF;
      break;
    case 'g':
      weight += __SG;
      break;
    case 'h':
      weight += __SH;
      break;
    case 'k':
      weight += __SK;
      break;
    case 'l':
      weight += __SL;
      break;
    case 'x':
      weight += __SX;
      break;
    case 'c':
      weight += __SC;
      break;
    case 'v':
      weight += __SV;
      break;
    case 'n':
      weight += __SN;
      break;
    case 'm':
      weight += __SM;
      break;
    case '[':
      weight += __Sksl;
      break;
    case ':':
      weight += __Sdt;
      break;
    case ',':
      weight += __Szp;
      break;
    case '.':
      weight += __Stch;
      break;
    }
  break;
  case 'd':
    switch(word2){
    case 'e':
      weight += __DE;
      break;
    case 'r':
      weight += __DR;
      break;
    case 't':
      weight += __DT;
      break;
    case 'y':
      weight += __DY;
      break;
    case 'u':
      weight += __DU;
      break;
    case 'i':
      weight += __DI;
      break;
    case 'o':
      weight += __DO;
      break;
    case 'p':
      weight += __DP;
      break;
    case 'a':
      weight += __DA;
      break;
    case 's':
      weight += __DS;
      break;
    case 'd':
      weight += __DD;
      break;
    case 'f':
      weight += __DF;
      break;
    case 'g':
      weight += __DG;
      break;
    case 'h':
      weight += __DH;
      break;
    case 'j':
      weight += __DJ;
      break;
    case 'k':
      weight += __DK;
      break;
    case 'z':
      weight += __DZ;
      break;
    case 'x':
      weight += __DX;
      break;
    case 'c':
      weight += __DC;
      break;
    case 'b':
      weight += __DB;
      break;
    case 'n':
      weight += __DN;
      break;
    case ':':
      weight += __Ddt;
      break;
    case ';':
      weight += __Dtz;
      break;
    case ',':
      weight += __Dzp;
      break;
    case '.':
      weight += __Dtch;
      break;
    }
  break;
  case 'f':
    switch(word2){
    case 'q':
      weight += __FQ;
      break;
    case 'w':
      weight += __FW;
      break;
    case 'e':
      weight += __FE;
      break;
    case 'r':
      weight += __FR;
      break;
    case 't':
      weight += __FT;
      break;
    case 'y':
      weight += __FY;
      break;
    case 'u':
      weight += __FU;
      break;
    case 'i':
      weight += __FI;
      break;
    case 'o':
      weight += __FO;
      break;
    case 'p':
      weight += __FP;
      break;
    case 'a':
      weight += __FA;
      break;
    case 'd':
      weight += __FD;
      break;
    case 'f':
      weight += __FF;
      break;
    case 'g':
      weight += __FG;
      break;
    case 'h':
      weight += __FH;
      break;
    case 'k':
      weight += __FK;
      break;
    case 'l':
      weight += __FL;
      break;
    case 'z':
      weight += __FZ;
      break;
    case 'x':
      weight += __FX;
      break;
    case 'c':
      weight += __FC;
      break;
    case 'v':
      weight += __FV;
      break;
    case 'b':
      weight += __FB;
      break;
    case 'n':
      weight += __FN;
      break;
    case '[':
      weight += __Fksl;
      break;
    case ':':
      weight += __Fdt;
      break;
    case ';':
      weight += __Ftz;
      break;
    case ',':
      weight += __Fzp;
      break;
    case '.':
      weight += __Ftch;
      break;
    }
  break;
  case 'g':
    switch(word2){
    case 'e':
      weight += __GE;
      break;
    case 'r':
      weight += __GR;
      break;
    case 't':
      weight += __GT;
      break;
    case 'y':
      weight += __GY;
      break;
    case 'u':
      weight += __GU;
      break;
    case 'i':
      weight += __GI;
      break;
    case 'o':
      weight += __GO;
      break;
    case 'a':
      weight += __GA;
      break;
    case 's':
      weight += __GS;
      break;
    case 'f':
      weight += __GF;
      break;
    case 'g':
      weight += __GG;
      break;
    case 'h':
      weight += __GH;
      break;
    case 'j':
      weight += __GJ;
      break;
    case 'k':
      weight += __GK;
      break;
    case 'l':
      weight += __GL;
      break;
    case 'z':
      weight += __GZ;
      break;
    case 'c':
      weight += __GC;
      break;
    case 'b':
      weight += __GB;
      break;
    case 'n':
      weight += __GN;
      break;
    case 'm':
      weight += __GM;
      break;
    case ':':
      weight += __Gdt;
      break;
    case ',':
      weight += __Gzp;
      break;
    case '.':
      weight += __Gtch;
      break;
    }
  break;
  case 'h':
    switch(word2){
    case 'e':
      weight += __HE;
      break;
    case 'r':
      weight += __HR;
      break;
    case 't':
      weight += __HT;
      break;
    case 'y':
      weight += __HY;
      break;
    case 'u':
      weight += __HU;
      break;
    case 'i':
      weight += __HI;
      break;
    case 'o':
      weight += __HO;
      break;
    case 'a':
      weight += __HA;
      break;
    case 's':
      weight += __HS;
      break;
    case 'd':
      weight += __HD;
      break;
    case 'f':
      weight += __HF;
      break;
    case 'g':
      weight += __HG;
      break;
    case 'h':
      weight += __HH;
      break;
    case 'j':
      weight += __HJ;
      break;
    case 'k':
      weight += __HK;
      break;
    case 'l':
      weight += __HL;
      break;
    case 'z':
      weight += __HZ;
      break;
    case 'x':
      weight += __HX;
      break;
    case 'c':
      weight += __HC;
      break;
    case 'v':
      weight += __HV;
      break;
    case 'b':
      weight += __HB;
      break;
    case 'n':
      weight += __HN;
      break;
    case 'm':
      weight += __HM;
      break;
    case '[':
      weight += __Hksl;
      break;
    case ';':
      weight += __Htz;
      break;
    case ',':
      weight += __Hzp;
      break;
    case '.':
      weight += __Htch;
      break;
    }
  break;
  case 'j':
    switch(word2){
    case 'q':
      weight += __JQ;
      break;
    case 'w':
      weight += __JW;
      break;
    case 'e':
      weight += __JE;
      break;
    case 'r':
      weight += __JR;
      break;
    case 't':
      weight += __JT;
      break;
    case 'y':
      weight += __JY;
      break;
    case 'u':
      weight += __JU;
      break;
    case 'i':
      weight += __JI;
      break;
    case 'o':
      weight += __JO;
      break;
    case 'p':
      weight += __JP;
      break;
    case 'a':
      weight += __JA;
      break;
    case 'd':
      weight += __JD;
      break;
    case 'g':
      weight += __JG;
      break;
    case 'h':
      weight += __JH;
      break;
    case 'j':
      weight += __JJ;
      break;
    case 'k':
      weight += __JK;
      break;
    case 'l':
      weight += __JL;
      break;
    case 'z':
      weight += __JZ;
      break;
    case 'x':
      weight += __JX;
      break;
    case 'c':
      weight += __JC;
      break;
    case 'v':
      weight += __JV;
      break;
    case 'b':
      weight += __JB;
      break;
    case 'n':
      weight += __JN;
      break;
    case '[':
      weight += __Jksl;
      break;
    case ':':
      weight += __Jdt;
      break;
    case ';':
      weight += __Jtz;
      break;
    case ',':
      weight += __Jzp;
      break;
    case '.':
      weight += __Jtch;
      break;
    }
  break;
  case 'k':
    switch(word2){
    case 'e':
      weight += __KE;
      break;
    case 'r':
      weight += __KR;
      break;
    case 't':
      weight += __KT;
      break;
    case 'y':
      weight += __KY;
      break;
    case 'u':
      weight += __KU;
      break;
    case 'i':
      weight += __KI;
      break;
    case 'o':
      weight += __KO;
      break;
    case 'a':
      weight += __KA;
      break;
    case 's':
      weight += __KS;
      break;
    case 'f':
      weight += __KF;
      break;
    case 'h':
      weight += __KH;
      break;
    case 'j':
      weight += __KJ;
      break;
    case 'k':
      weight += __KK;
      break;
    case 'z':
      weight += __KZ;
      break;
    case 'x':
      weight += __KX;
      break;
    case 'c':
      weight += __KC;
      break;
    case 'b':
      weight += __KB;
      break;
    case 'n':
      weight += __KN;
      break;
    case 'm':
      weight += __KM;
      break;
    case ':':
      weight += __Kdt;
      break;
    case ';':
      weight += __Ktz;
      break;
    case ',':
      weight += __Kzp;
      break;
    case '.':
      weight += __Ktch;
      break;
    }
  break;
  case 'l':
    switch(word2){
    case 'w':
      weight += __LW;
      break;
    case 'e':
      weight += __LE;
      break;
    case 'r':
      weight += __LR;
      break;
    case 't':
      weight += __LT;
      break;
    case 'y':
      weight += __LY;
      break;
    case 'u':
      weight += __LU;
      break;
    case 'i':
      weight += __LI;
      break;
    case 'o':
      weight += __LO;
      break;
    case 'p':
      weight += __LP;
      break;
    case 'a':
      weight += __LA;
      break;
    case 's':
      weight += __LS;
      break;
    case 'd':
      weight += __LD;
      break;
    case 'f':
      weight += __LF;
      break;
    case 'g':
      weight += __LG;
      break;
    case 'h':
      weight += __LH;
      break;
    case 'j':
      weight += __LJ;
      break;
    case 'k':
      weight += __LK;
      break;
    case 'l':
      weight += __LL;
      break;
    case 'z':
      weight += __LZ;
      break;
    case 'c':
      weight += __LC;
      break;
    case 'v':
      weight += __LV;
      break;
    case 'b':
      weight += __LB;
      break;
    case 'n':
      weight += __LN;
      break;
    case 'm':
      weight += __LM;
      break;
    case ';':
      weight += __Ltz;
      break;
    case ',':
      weight += __Lzp;
      break;
    case '.':
      weight += __Ltch;
      break;
    }
  break;
  case 'z':
    switch(word2){
    case 'e':
      weight += __ZE;
      break;
    case 'r':
      weight += __ZR;
      break;
    case 't':
      weight += __ZT;
      break;
    case 'y':
      weight += __ZY;
      break;
    case 'u':
      weight += __ZU;
      break;
    case 'o':
      weight += __ZO;
      break;
    case 'p':
      weight += __ZP;
      break;
    case 'a':
      weight += __ZA;
      break;
    case 'd':
      weight += __ZD;
      break;
    case 'h':
      weight += __ZH;
      break;
    case 'k':
      weight += __ZK;
      break;
    case 'l':
      weight += __ZL;
      break;
    case 'x':
      weight += __ZX;
      break;
    case 'c':
      weight += __ZC;
      break;
    case 'v':
      weight += __ZV;
      break;
    case 'n':
      weight += __ZN;
      break;
    case '[':
      weight += __Zksl;
      break;
    case ':':
      weight += __Zdt;
      break;
    case ';':
      weight += __Ztz;
      break;
    case ',':
      weight += __Zzp;
      break;
    case '.':
      weight += __Ztch;
      break;
    }
  break;
  case 'x':
    switch(word2){
    case 'e':
      weight += __XE;
      break;
    case 'r':
      weight += __XR;
      break;
    case 't':
      weight += __XT;
      break;
    case 'y':
      weight += __XY;
      break;
    case 'p':
      weight += __XP;
      break;
    case 'a':
      weight += __XA;
      break;
    case 'f':
      weight += __XF;
      break;
    case 'j':
      weight += __XJ;
      break;
    case 'c':
      weight += __XC;
      break;
    case 'b':
      weight += __XB;
      break;
    case 'n':
      weight += __XN;
      break;
    case 'm':
      weight += __XM;
      break;
    }
  break;
  case 'c':
    switch(word2){
    case 'q':
      weight += __CQ;
      break;
    case 'e':
      weight += __CE;
      break;
    case 'r':
      weight += __CR;
      break;
    case 't':
      weight += __CT;
      break;
    case 'y':
      weight += __CY;
      break;
    case 'u':
      weight += __CU;
      break;
    case 'i':
      weight += __CI;
      break;
    case 'o':
      weight += __CO;
      break;
    case 'a':
      weight += __CA;
      break;
    case 's':
      weight += __CS;
      break;
    case 'd':
      weight += __CD;
      break;
    case 'f':
      weight += __CF;
      break;
    case 'g':
      weight += __CG;
      break;
    case 'h':
      weight += __CH;
      break;
    case 'j':
      weight += __CJ;
      break;
    case 'k':
      weight += __CK;
      break;
    case 'l':
      weight += __CL;
      break;
    case 'z':
      weight += __CZ;
      break;
    case 'x':
      weight += __CX;
      break;
    case 'c':
      weight += __CC;
      break;
    case 'v':
      weight += __CV;
      break;
    case 'b':
      weight += __CB;
      break;
    case 'n':
      weight += __CN;
      break;
    case 'm':
      weight += __CM;
      break;
    case '[':
      weight += __Cksl;
      break;
    case ',':
      weight += __Czp;
      break;
    case '.':
      weight += __Ctch;
      break;
    }
  break;
  case 'v':
    switch(word2){
    case 'e':
      weight += __VE;
      break;
    case 'r':
      weight += __VR;
      break;
    case 't':
      weight += __VT;
      break;
    case 'y':
      weight += __VY;
      break;
    case 'i':
      weight += __VI;
      break;
    case 'o':
      weight += __VO;
      break;
    case 'a':
      weight += __VA;
      break;
    case 's':
      weight += __VS;
      break;
    case 'f':
      weight += __VF;
      break;
    case 'g':
      weight += __VG;
      break;
    case 'h':
      weight += __VH;
      break;
    case 'j':
      weight += __VJ;
      break;
    case 'k':
      weight += __VK;
      break;
    case 'z':
      weight += __VZ;
      break;
    case 'c':
      weight += __VC;
      break;
    case 'v':
      weight += __VV;
      break;
    case 'b':
      weight += __VB;
      break;
    case 'm':
      weight += __VM;
      break;
    case ',':
      weight += __Vzp;
      break;
    case '.':
      weight += __Vtch;
      break;
    }
  break;
  case 'b':
    switch(word2){
    case 'q':
      weight += __BQ;
      break;
    case 'w':
      weight += __BW;
      break;
    case 'e':
      weight += __BE;
      break;
    case 'r':
      weight += __BR;
      break;
    case 't':
      weight += __BT;
      break;
    case 'y':
      weight += __BY;
      break;
    case 'u':
      weight += __BU;
      break;
    case 'i':
      weight += __BI;
      break;
    case 'o':
      weight += __BO;
      break;
    case 'p':
      weight += __BP;
      break;
    case 'a':
      weight += __BA;
      break;
    case 's':
      weight += __BS;
      break;
    case 'd':
      weight += __BD;
      break;
    case 'f':
      weight += __BF;
      break;
    case 'g':
      weight += __BG;
      break;
    case 'h':
      weight += __BH;
      break;
    case 'j':
      weight += __BJ;
      break;
    case 'k':
      weight += __BK;
      break;
    case 'l':
      weight += __BL;
      break;
    case 'z':
      weight += __BZ;
      break;
    case 'x':
      weight += __BX;
      break;
    case 'c':
      weight += __BC;
      break;
    case 'v':
      weight += __BV;
      break;
    case 'b':
      weight += __BB;
      break;
    case 'n':
      weight += __BN;
      break;
    case '[':
      weight += __Bksl;
      break;
    case ':':
      weight += __Bdt;
      break;
    case ';':
      weight += __Btz;
      break;
    case ',':
      weight += __Bzp;
      break;
    case '.':
      weight += __Btch;
      break;
    }
  break;
  case 'n':
    switch(word2){
    case 'e':
      weight += __NE;
      break;
    case 'r':
      weight += __NR;
      break;
    case 'y':
      weight += __NY;
      break;
    case 'u':
      weight += __NU;
      break;
    case 'i':
      weight += __NI;
      break;
    case 'o':
      weight += __NO;
      break;
    case 'a':
      weight += __NA;
      break;
    case 's':
      weight += __NS;
      break;
    case 'd':
      weight += __ND;
      break;
    case 'f':
      weight += __NF;
      break;
    case 'g':
      weight += __NG;
      break;
    case 'h':
      weight += __NH;
      break;
    case 'j':
      weight += __NJ;
      break;
    case 'k':
      weight += __NK;
      break;
    case 'l':
      weight += __NL;
      break;
    case 'z':
      weight += __NZ;
      break;
    case 'x':
      weight += __NX;
      break;
    case 'c':
      weight += __NC;
      break;
    case 'v':
      weight += __NV;
      break;
    case 'b':
      weight += __NB;
      break;
    case 'n':
      weight += __NN;
      break;
    case 'm':
      weight += __NM;
      break;
    case ':':
      weight += __Ndt;
      break;
    case ';':
      weight += __Ntz;
      break;
    case ',':
      weight += __Nzp;
      break;
    case '.':
      weight += __Ntch;
      break;
    }
  break;
  case 'm':
    switch(word2){
    case 'w':
      weight += __MW;
      break;
    case 'e':
      weight += __ME;
      break;
    case 'r':
      weight += __MR;
      break;
    case 't':
      weight += __MT;
      break;
    case 'y':
      weight += __MY;
      break;
    case 'u':
      weight += __MU;
      break;
    case 'i':
      weight += __MI;
      break;
    case 'o':
      weight += __MO;
      break;
    case 'p':
      weight += __MP;
      break;
    case 'a':
      weight += __MA;
      break;
    case 's':
      weight += __MS;
      break;
    case 'l':
      weight += __ML;
      break;
    case 'z':
      weight += __MZ;
      break;
    case 'x':
      weight += __MX;
      break;
    case 'c':
      weight += __MC;
      break;
    case 'v':
      weight += __MV;
      break;
    case 'b':
      weight += __MB;
      break;
    case 'm':
      weight += __MM;
      break;
    case ':':
      weight += __Mdt;
      break;
    case ',':
      weight += __Mzp;
      break;
    case '.':
      weight += __Mtch;
      break;
    }
  break;
  case '[':
    switch(word2){
    case 'e':
      weight += __kslE;
      break;
    case 'y':
      weight += __kslY;
      break;
    case 'd':
      weight += __kslD;
      break;
    case 'f':
      weight += __kslF;
      break;
    case 'h':
      weight += __kslH;
      break;
    case 'j':
      weight += __kslJ;
      break;
    case 'k':
      weight += __kslK;
      break;
    case 'b':
      weight += __kslB;
      break;
    case ',':
      weight += __kslzp;
      break;
    case '.':
      weight += __ksltch;
      break;
    }
  break;
  case ']':
    switch(word2){
    case 't':
      weight += __kspT;
      break;
    case 'z':
      weight += __kspZ;
      break;
    }
  break;
  case '{':
    switch(word2){
    case 'j':
      weight += __fslJ;
      break;
    }
  break;
  case ';':
    switch(word2){
    case 'e':
      weight += __tzE;
      break;
    case 'r':
      weight += __tzR;
      break;
    case 't':
      weight += __tzT;
      break;
    case 'y':
      weight += __tzY;
      break;
    case 'f':
      weight += __tzF;
      break;
    case 'j':
      weight += __tzJ;
      break;
    case 'l':
      weight += __tzL;
      break;
    case 'b':
      weight += __tzB;
      break;
    case ',':
      weight += __tzzp;
      break;
    }
  break;
  case '<':
    switch(word2){
    case 'e':
      weight += __tslE;
      break;
    case 't':
      weight += __tslT;
      break;
    case 'h':
      weight += __tslH;
      break;
    case 'j':
      weight += __tslJ;
      break;
    case 'b':
      weight += __tslB;
      break;
    }
  break;
  case ',':
    switch(word2){
    case 'e':
      weight += __zpE;
      break;
    case 'r':
      weight += __zpR;
      break;
    case 't':
      weight += __zpT;
      break;
    case 'y':
      weight += __zpY;
      break;
    case 'o':
      weight += __zpO;
      break;
    case 's':
      weight += __zpS;
      break;
    case 'd':
      weight += __zpD;
      break;
    case 'f':
      weight += __zpF;
      break;
    case 'h':
      weight += __zpH;
      break;
    case 'j':
      weight += __zpJ;
      break;
    case 'k':
      weight += __zpK;
      break;
    case 'z':
      weight += __zpZ;
      break;
    case 'c':
      weight += __zpC;
      break;
    case 'b':
      weight += __zpB;
      break;
    case ']':
      weight += __zpksp;
      break;
    case ',':
      weight += __zpzp;
      break;
    case '.':
      weight += __zptch;
      break;
    }
  break;
  case '.':
    switch(word2){
    case 'o':
      weight += __tchO;
      break;
    case 'l':
      weight += __tchL;
      break;
    case 'x':
      weight += __tchX;
      break;
    case 'c':
      weight += __tchC;
      break;
    case 'n':
      weight += __tchN;
      break;
    case ',':
      weight += __tchzp;
      break;
    case '.':
      weight += __tchtch;
      break;
    }
  break;
}
return weight;
}
