/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Art Vodolazsky <burivuh@crew.org.ru>
 *  some modifycations and code clean-up by Alphex Kaanoken <kaanoken@crew.org.ru>
 *
 *  (c) Crew IT Research Labs 2004
 *
 */

#include "error.h"
#include "utils.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#define DBG_OTP  "[XXX]: "

status eManager_manage (eReport * ereport, status eOutput)
{

	char *eString = NULL;
	int lw = 1;

	if (ereport->eString == NULL)
		return ERROR;

	lw += strlen (ereport->eString) + strlen (DBG_OTP);

	eString = (char *) malloc (lw);
	if (eString == NULL)
		return ERROR;
	eString = memset (eString, NULLSYM, lw);

	switch (ereport->eType) {
		case ERROR:
			strcat (eString, "[ERR]: ");
			break;
		case WARNING:
			strcat (eString, "[WRN]: ");
			break;
		case DEBUG:
			strcat (eString, "[DBG]: ");
			break;
		default:
			break;
	}

	strcat (eString, ereport->eString);

	//sending string to specified output

	switch (eOutput) {
		case STDOUT:
			fprintf (stdout, eString, ereport->eData);
			break;
		case LOG:
			break;
		default:
			break;
	}

	free (eString);

	return SUCCESS;
}

eReport *eReport_make (status eType, char *eString, void *eData, char *_FILE, num _LINE)
{
	eReport *ereport = NULL;

	ereport = (eReport *) _malloc (sizeof (eReport));

	ereport->eString = (char *) _malloc (sizeof (char) * strlen (eString) + 1);

	memset (ereport->eString, NULLSYM, strlen (eString) + 1);
	memcpy (ereport->eString, eString, strlen (eString));
	ereport->eType = eType;
	if (eData != NULL)
		memcpy (&ereport->eData, eData, sizeof (void *));
	else
		ereport->eData = NULLSYM;
	ereport->eLoc.filename = _FILE;
	ereport->eLoc.linenum = _LINE;

	return ereport;
}

status eReport_free (eReport * ereport)
{
	if (ereport)
		free (ereport);
	else
		return WARNING;
	return SUCCESS;
}

status eReport_send (status eType, char *eString, void *eData, char *_FILE, num _LINE)
{
	eReport *ereport = NULL;

	ereport = eReport_make (eType, eString, eData, _FILE, _LINE);
	if (!ereport)
		return BAD_ALLOC;
	if (SUCCESS != eManager_manage (ereport, STDOUT)) {
		eReport_free (ereport);
		return BAD_ALLOC;
	}
	if (SUCCESS != eReport_free (ereport))
		return WARNING;

	return SUCCESS;
}
