
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Art Vodolazsky <burivuh@crew.org.ru>
 */

#ifndef _XKEYLOGGER_H
#define _XKEYLOGGER_H

#include "types.h"
#include "xLibrary.h"

//BufferActions
#define KLB_NO_ACTION			0	//modifier, function etc
#define KLB_ADD_SYM				1	//alpha
#define KLB_DEL_SYM_RIGHT	2	//del, bkspace etc
#define KLB_DEL_SYM_LEFT	3	// -//-
#define KLB_MOVE_RIGHT		4	//cursor right
#define KLB_MOVE_LEFT			5	//cursor left
#define KLB_FLUSH					6	//word end (space etc)
#define KLB_CLEAR					7	//Home, End etc
#define KLB_SWITCH_MODE		8	//Insert
#define KLB_SWITCH_LANG		9

//mode
#define KLB_INSERT_MODE		10
#define KLB_REPLACE_MODE	11

//bind actions
#define BA_NONE						0
#define BA_CHANGE_WORD		1
#define BA_CHANGE_STRING	2
#define BA_CHANGE_MODE		3
#define BA_CHANGE_SELECT	4

typedef struct {
	xString *buffer;
	num iterator;
	status mode;
} xKeyLogBuffer;

xKeyLogBuffer *xKeyLogBuffer_init ();
status xKeyLogBuffer_free (xKeyLogBuffer * toFree);
status xKeyLogBuffer_clear (xKeyLogBuffer * toClear);

char *xKeyLogManager_getStringFromEventQueue (xKeyLogBuffer * buffer,
					      xEvent * xevent,
					      xIFocus * xifocus,
					      xString * xword,
					      xString * xstring,
					      xSwitchLang * xswlang, xBindingTable * xbtable);

status xKeyLogManager_getBufferAction (xEvent * xevent);

status xKeyLogManager_getBindAction (xEvent * xevent, xBindingTable * xbtable);

status xKeyLogManager_flushBuffer (xKeyLogBuffer * buffer, xString * xstring);

status xKeyLogManager_bind (xBindingTable * xbtable, status key, status modifier, status action);

#endif /* _XKEYLOGGER_H */
